@@ -30,15 +30,15 @@ typedef enum
         EMBED_CHROME_MENUBARON = 1 << 1,
         EMBED_CHROME_TOOLBARON = 1 << 2,
 	EMBED_CHROME_BOOKMARKSBARON = 1 << 3,
-        EMBED_CHROME_STATUSBARON = 1 << 4,
-        EMBED_CHROME_WINDOWRAISED = 1 << 5,
-        EMBED_CHROME_WINDOWLOWERED = 1 << 6,
-        EMBED_CHROME_CENTERSCREEN = 1 << 7,
-	EMBED_CHROME_OPENASDIALOG = 1 << 8,
-	EMBED_CHROME_OPENASCHROME = 1 << 9,
-	EMBED_CHROME_OPENASPOPUP = 1 << 10,
-	EMBED_CHROME_OPENASFULLSCREEN = 1 << 11,
-	EMBED_CHROME_PPVIEWTOOLBARON = 1 << 12,
+	EMBED_CHROME_BOOKMARKSBAR_DEFAULT = 1 << 4,
+        EMBED_CHROME_STATUSBARON = 1 << 5,
+        EMBED_CHROME_WINDOWRAISED = 1 << 6,
+        EMBED_CHROME_WINDOWLOWERED = 1 << 7,
+        EMBED_CHROME_CENTERSCREEN = 1 << 8,
+	EMBED_CHROME_OPENASDIALOG = 1 << 9,
+	EMBED_CHROME_OPENASCHROME = 1 << 10,
+	EMBED_CHROME_OPENASPOPUP = 1 << 11,
+	EMBED_CHROME_PPVIEWTOOLBARON = 1 << 12
 } EmbedChromeMask;
 
 G_END_DECLS
