@@ -26,22 +26,16 @@
 
 G_BEGIN_DECLS
 
-typedef struct
-{
-        const char *encoding;
-        gpointer data;
-} EncodingMenuData;
-
 void ephy_embed_utils_save			(GtkWidget *window,
 						 const char *default_dir_pref,
 						 gboolean ask_dest,
 						 gboolean with_content,
 						 EphyEmbedPersist *persist);
 
-void ephy_embed_utils_build_charsets_submenu    (BonoboUIComponent *ui_component,
+void ephy_embed_utils_build_encodings_submenu   (BonoboUIComponent *ui_component,
 						 const char *path,
 						 BonoboUIVerbFn fn,
-						 gpointer data);
+						 gpointer view);
 
 void ephy_embed_utils_nohandler_dialog_run      (GtkWidget *parent);
 
