@@ -388,12 +388,13 @@ favicon_download_completed_cb (EphyEmbedPersist *persist,
 {
 	char *url;
 
-	url = g_object_get_data (G_OBJECT (persist), "url"),
+	url = g_object_get_data (G_OBJECT (persist), "url");
+	g_return_if_fail (url != NULL);
+
+	g_signal_emit (G_OBJECT (cache), ephy_favicon_cache_signals[CHANGED], 0, url);
 
 	g_hash_table_remove (cache->priv->downloads_hash, url);
 	g_object_unref (persist);
-
-	g_signal_emit (G_OBJECT (cache), ephy_favicon_cache_signals[CHANGED], 0, url);
 }
 
 static void
@@ -402,7 +403,7 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 			     const char *filename)
 {
 	EphyEmbedPersist *persist;
-	const char *dest;
+	char *dest;
 
 	LOG ("Download favicon: %s", favicon_url)
 
@@ -419,6 +420,8 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 	ephy_embed_persist_set_source   (persist, favicon_url);
 	ephy_embed_persist_set_dest     (persist, dest);
 
+	g_free (dest);
+
 	g_object_set_data_full (G_OBJECT (persist), "url",
 				g_strdup (favicon_url), g_free);
 
@@ -427,10 +430,10 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 			  G_CALLBACK (favicon_download_completed_cb),
 			  cache);
 
-	ephy_embed_persist_save (persist);
-
 	g_hash_table_insert (cache->priv->downloads_hash,
 			     g_strdup (favicon_url), persist);
+
+	ephy_embed_persist_save (persist);
 }
 
 GdkPixbuf *
@@ -440,9 +443,11 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 	GTime now;
 	EphyNode *icon;
 	GValue value = { 0, };
-	const char *pix_file;
+	char *pix_file;
 	GdkPixbuf *pixbuf;
 
+	if (url == NULL) return NULL;
+
 	now = time (NULL);
 
 	g_static_rw_lock_reader_lock (cache->priv->icons_hash_lock);
@@ -479,6 +484,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 	g_value_set_int (&value, now);
 	ephy_node_set_property (icon, EPHY_NODE_FAVICON_PROP_LAST_USED,
 			        &value);
+	g_value_unset (&value);
 
 	if (g_hash_table_lookup (cache->priv->downloads_hash, url) != NULL)
 	{
@@ -491,8 +497,6 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		 ephy_node_get_property_string (icon, EPHY_NODE_FAVICON_PROP_FILENAME),
 		 NULL);
 
-	g_hash_table_lookup (cache->priv->icons_hash, url);
-
 	LOG ("Create pixbuf for %s", pix_file)
 
 	pixbuf = gdk_pixbuf_new_from_file (pix_file, NULL);
@@ -507,7 +511,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		pixbuf = scaled;
 	}
 
+	g_free (pix_file);
+
 	return pixbuf;
 }
-
-
