@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: find-dialog.c,v 1.13 2003/06/27 23:10:53 chpe Exp $
+ *  $Id: find-dialog.c,v 1.14 2003/06/29 17:53:46 chpe Exp $
  */
 
 #include "find-dialog.h"
@@ -206,7 +206,7 @@ ensure_constructed (FindDialog *dialog)
 }
 
 static void
-sync_page_change (EphyEmbed *embed, FindDialog *dialog)
+sync_page_change (EphyEmbed *embed, const char *address, FindDialog *dialog)
 {
 	g_return_if_fail (IS_EPHY_EMBED (embed));
 
