@@ -22,13 +22,7 @@
 
 #include "ephy-embed-shell.h"
 #include "GlobalHistory.h"
-#include "nsCOMPtr.h"
-#include "nsISupportsArray.h"
-#include "nsIFactory.h"
-#include "nsIServiceManager.h"
 #include "nsString.h"
-#include "nsReadableUtils.h"
-#include "nsIRequestObserver.h"
 
 NS_IMPL_ISUPPORTS2(MozGlobalHistory, nsIGlobalHistory, nsIBrowserHistory)
 
@@ -96,6 +90,13 @@ NS_IMETHODIMP MozGlobalHistory::GetLastPageVisited(char **aLastPageVisited)
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+#if MOZILLA_SNAPSHOT > 8
+NS_IMETHODIMP MozGlobalHistory::SetLastPageVisited(const char *aLastPageVisited)
+{
+        return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
 NS_IMETHODIMP MozGlobalHistory::HidePage(const char *url)
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
