@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ProgressListener.cpp,v 1.9 2003/05/25 18:59:59 xan Exp $
+ *  $Id: ProgressListener.cpp,v 1.16 2003/06/22 00:38:04 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,16 +23,9 @@
 #endif
 
 #include "ProgressListener.h"
-
 #include "ephy-file-helpers.h"
 #include "mozilla-embed-persist.h"
-
-#include <unistd.h>
-#include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-i18n.h>
-
 #include "nsXPIDLString.h"
-#include "nsIMIMEInfo.h"
 #include "nsCOMPtr.h"
 
 static void
@@ -65,19 +58,19 @@ GProgressListener::~GProgressListener ()
 }
 
 NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
-					      nsIDOMWindow *aParent, 
-					      nsIURI *aURI,
-					      nsIFile *aFile,
-					      DownloadAction aAction,
-					      EphyEmbedPersist *ephyPersist,
-					      PRBool noDialog,
-					      PRInt64 aTimeDownloadStarted)
+					     nsIDOMWindow *aParent, 
+					     nsIURI *aURI,
+					     nsIFile *aFile,
+					     DownloadAction aAction,
+					     EphyEmbedPersist *ephyPersist,
+					     PRBool Dialog,
+					     PRInt64 aTimeDownloadStarted)
 {
 	nsresult rv;
 	/* fill in download details */
 	mAction = aAction;
 	mParent = aParent;
-	mNoDialog = noDialog;
+	mDialog = Dialog;
 	mUri = aURI;
 	mFile = aFile;
 	mPersist = aPersist;
@@ -90,63 +83,61 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	/* pick up progress messages */
 	mPersist->SetProgressListener (this);
 
-	/* done */
 	return rv;
 }
 
 NS_METHOD GProgressListener::PrivateInit (void)
 {
-	nsresult rv;
-
-	/* setup this download */
-	mInterval            = 500000;     /* in microsecs == 500ms == 0.5s */
+	mInterval            = 500000;     /* in microsecs, 0.5s */
 	mPriorKRate          = 0;
 	mRateChanges         = 0;
 	mRateChangeLimit     = 2;          /* only update rate every second */
 	mIsPaused            = PR_FALSE;
 	mAbort               = PR_FALSE;
-	PRInt64 now          = PR_Now ();
-	mLastUpdate          = now;
+	mStartTime           = PR_Now ();
+	mLastUpdate          = mStartTime;
+	mElapsed             = 0;
 
-//	mStartTime           = (mTimeDownloadStarted != 0 ? 
-//				mTimeDownloadStarted : now);
-	mStartTime           = now; //Stupid mozilla race condition
-
-	mElapsed             = now - mStartTime;
-
-	if (!mNoDialog)
+	if (mDialog)
 	{
 		gchar *filename, *source, *dest;
 		nsAutoString uTmp;
 		nsCAutoString cTmp;
+		nsresult rv;
 
 		rv = mFile->GetLeafName (uTmp);
+		if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 		filename = g_strdup (NS_ConvertUCS2toUTF8(uTmp).get());
 
 		rv = mFile->GetPath (uTmp);
+		if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 		dest = g_strdup (NS_ConvertUCS2toUTF8(uTmp).get());
 
 		rv = mUri->GetSpec (cTmp);
+		if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 		source = g_strdup (cTmp.get());
-		DownloaderView *dv;
-		dv = ephy_embed_shell_get_downloader_view (embed_shell);
-		downloader_view_add_download (dv, filename, source, dest, (gpointer)this);
-		g_signal_connect (G_OBJECT (dv), 
+
+		mDownloaderView = ephy_embed_shell_get_downloader_view (embed_shell);
+		downloader_view_add_download (mDownloaderView, filename, source,
+					      dest, (gpointer)this);
+		g_free (source);
+		g_free (dest);
+		g_free (filename);
+
+		g_signal_connect (G_OBJECT (mDownloaderView), 
 				  "download_remove",
 				  G_CALLBACK (download_remove_cb),
 				  this);
-		g_signal_connect (G_OBJECT (dv), 
+		g_signal_connect (G_OBJECT (mDownloaderView), 
 				  "download_pause",
 				  G_CALLBACK (download_pause_cb),
 				  this);
-		g_signal_connect (G_OBJECT (dv), 
+		g_signal_connect (G_OBJECT (mDownloaderView), 
 				  "download_resume",
 				  G_CALLBACK (download_resume_cb),
 				  this);
-		mDownloaderView = dv;
 	}
 
-	/* done */
 	return NS_OK;
 }
 
@@ -173,7 +164,7 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
 				  ACTION_SAVEFORHELPER : ACTION_NONE;
 		}
 	}
-        mNoDialog = 0;
+        mDialog = TRUE;
 
         return PrivateInit();
 }
@@ -181,7 +172,7 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
 NS_IMETHODIMP GProgressListener::Open(nsIDOMWindow *aParent)
 {
         mParent = aParent;
-        mNoDialog = 0;
+        mDialog = TRUE;
 
         return NS_OK;
 }
@@ -274,12 +265,12 @@ NS_IMETHODIMP GProgressListener::GetDialog(nsIDOMWindow * *aDialog)
 /* attribute PRBool cancelDownloadOnClose; */
 NS_IMETHODIMP  GProgressListener::GetCancelDownloadOnClose(PRBool *aCancelDownloadOnClose)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP  GProgressListener::SetCancelDownloadOnClose(PRBool aCancelDownloadOnClose)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP GProgressListener::LaunchHandler (PersistHandlerInfo *handler)
@@ -316,14 +307,22 @@ NS_IMETHODIMP GProgressListener::LaunchHandler (PersistHandlerInfo *handler)
  *		       in unsigned long aStatus);
  */
 NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
-						 nsIRequest *aRequest,
-						 PRUint32 aStateFlags,
-						 PRUint32 aStatus)
+						nsIRequest *aRequest,
+						PRUint32 aStateFlags,
+						PRUint32 aStatus)
 {
 	if (mAbort) return NS_ERROR_FAILURE;
 
 	if (aStateFlags & nsIWebProgressListener::STATE_STOP)
 	{
+
+		if (mDialog)
+		{
+			downloader_view_set_download_status (mDownloaderView, 
+							     DOWNLOAD_STATUS_COMPLETED, 
+							     (gpointer)this);
+		}
+
 		switch (mAction)
                 {
 		case ACTION_SAVEFORHELPER:
@@ -353,18 +352,10 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
 			}
 		
 			mozilla_embed_persist_completed
-				(MOZILLA_EMBED_PERSIST (mEphyPersist));	
-		}
-		
-		if (!mNoDialog)
-		{
-			downloader_view_set_download_status (mDownloaderView, 
-							     DOWNLOAD_STATUS_COMPLETED, 
-							     (gpointer)this);
-		}
+				(MOZILLA_EMBED_PERSIST (mEphyPersist));
+		}		
 	}
 
-	/* done */
 	return NS_OK;
 }
 
@@ -388,7 +379,7 @@ NS_IMETHODIMP GProgressListener::
 
 	/* FIXME maxsize check here */
 
-	if (mNoDialog) return NS_OK;
+	if (!mDialog) return NS_OK;
 
 	mRequest = aRequest;
 
@@ -401,9 +392,9 @@ NS_IMETHODIMP GProgressListener::
 	{
 		return NS_OK;
 	}
+	mLastUpdate = now;
 
 	/* compute elapsed time */
-	mLastUpdate = now;
 	mElapsed = now - mStartTime;
 
 	/* compute size done */
@@ -425,51 +416,46 @@ NS_IMETHODIMP GProgressListener::
 	PRInt64 currentRate;
 	if (mElapsed)
 	{
-		currentRate = ((PRInt64)(aCurTotalProgress)) * 1000000 / 
-			mElapsed;
+		currentRate = ((PRInt64)(aCurTotalProgress)) * 1000000 / mElapsed;
 	}
 	else
 	{
 		currentRate = 0;
 	}		
 
-	if (!mIsPaused)
+	if (!mIsPaused && currentRate)
 	{
-		if (currentRate)
+		PRFloat64 currentKRate = ((PRFloat64)currentRate)/1024;
+		if (currentKRate != mPriorKRate)
 		{
-			PRFloat64 currentKRate = ((PRFloat64)currentRate)/1024;
-			if (currentKRate != mPriorKRate)
+			if (mRateChanges++ == mRateChangeLimit)
 			{
-				if (mRateChanges++ == mRateChangeLimit)
-				{
-					mPriorKRate = currentKRate;
-					mRateChanges = 0;
-				}
-				else
-				{
-					currentKRate = mPriorKRate;
-				}
+				mPriorKRate = currentKRate;
+				mRateChanges = 0;
 			}
 			else
 			{
-				mRateChanges = 0;
+				currentKRate = mPriorKRate;
 			}
-			
-			 speed = currentKRate;
 		}
+		else
+		{
+			mRateChanges = 0;
+		}
+
+		speed = currentKRate;
 	}
 	
 	/* compute time remaining */
 	gint remaining = -1;
 	if (currentRate && (aMaxTotalProgress > 0))
-	 {
-		 remaining = 
-			 (gint)((aMaxTotalProgress - aCurTotalProgress)
-				/currentRate +.5);
+	{
+		 remaining = (gint)((aMaxTotalProgress - aCurTotalProgress)
+				 /currentRate + 0.5);
 	}
 
 	downloader_view_set_download_progress (mDownloaderView,
-					       mElapsed,
+					       mElapsed / 1000000,
 					       remaining,
 					       speed,
 					       totalKBytes,
@@ -477,7 +463,6 @@ NS_IMETHODIMP GProgressListener::
 					       progress,
 					       (gpointer)this);
 
-	/* done */
 	return NS_OK;
 }
 
@@ -486,7 +471,7 @@ NS_IMETHODIMP GProgressListener::
 			OnLocationChange(nsIWebProgress *aWebProgress,
 					 nsIRequest *aRequest, nsIURI *location)
 {
-    return NS_OK;
+	return NS_OK;
 }
 
 /* void onStatusChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsresult aStatus, in wstring aMessage); */
@@ -495,7 +480,7 @@ NS_IMETHODIMP GProgressListener::
 				       nsIRequest *aRequest, nsresult aStatus,
 				       const PRUnichar *aMessage)
 {
-    return NS_OK;
+	return NS_OK;
 }
 
 /* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
@@ -504,7 +489,7 @@ NS_IMETHODIMP GProgressListener::
 					 nsIRequest *aRequest,
 					 PRUint32 state)
 {
-    return NS_OK;
+	return NS_OK;
 }
 
 //---------------------------------------------------------------------------
@@ -602,15 +587,8 @@ nsresult GProgressListener::Resume (void)
 nsresult GProgressListener::Abort (void)
 {
 	PRBool notify;
-	
 	notify = (mAction == ACTION_OBJECT_NOTIFY);
         mAction = ACTION_NONE;
-
-        if (mIsPaused)
-        {
-                Resume ();
-        }
-
         mAbort = PR_TRUE;
 
         if (mObserver)
