@@ -37,6 +37,7 @@
 #include <time.h>
 #include <libgnome/gnome-i18n.h>
 #include <string.h>
+#include "nsBuildID.h"
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsIPrefService.h>
@@ -46,8 +47,6 @@
 #include <nsIProtocolProxyService.h>
 #include <nsIJVMManager.h>
 #include <nsIAtom.h>
-#include <nsICharsetConverterManager.h>
-#include <nsICharsetConverterManager2.h>
 #include <nsIFontList.h>
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
@@ -103,32 +102,27 @@ struct
 }
 encodings[] =
 { 
+	/* translators: access keys need only be unique within the same LG_group */
 	{ N_("Arabic (_IBM-864)"),                  "IBM864",                LG_ARABIC },
-	{ N_("Arabic (I_BM-864-I)"),                "IBM864i",               LG_ARABIC },
 	{ N_("Arabic (ISO-_8859-6)"),               "ISO-8859-6",            LG_ARABIC },
-	{ N_("Arabic (I_SO-8859-6-E)"),             "ISO-8859-6-E",          LG_ARABIC },
-	{ N_("Arabic (IS_O-8859-6-I)"),             "ISO-8859-6-I",          LG_ARABIC },
 	{ N_("Arabic (_MacArabic)"),                "x-mac-arabic",          LG_ARABIC },
 	{ N_("Arabic (_Windows-1256)"),             "windows-1256",          LG_ARABIC },
-	{ N_("_Armenian (ARMSCII-8)"),              "armscii-8",             LG_OTHER },
 	{ N_("Baltic (_ISO-8859-13)"),              "ISO-8859-13",           LG_BALTIC },
 	{ N_("Baltic (I_SO-8859-4)"),               "ISO-8859-4",            LG_BALTIC },
 	{ N_("Baltic (_Windows-1257)"),             "windows-1257",          LG_BALTIC },
-	{ N_("_Celtic (ISO-8859-14)"),              "ISO-8859-14",           LG_OTHER },
 	{ N_("Central European (_IBM-852)"),        "IBM852",                LG_CENTRAL_EUROPEAN },
 	{ N_("Central European (I_SO-8859-2)"),     "ISO-8859-2",	     LG_CENTRAL_EUROPEAN },
 	{ N_("Central European (_MacCE)"),          "x-mac-ce",              LG_CENTRAL_EUROPEAN },
 	{ N_("Central European (_Windows-1250)"),   "windows-1250",          LG_CENTRAL_EUROPEAN },
+	{ N_("Croatian (Mac_Croatian)"),            "x-mac-croatian",        LG_CENTRAL_EUROPEAN },
 	{ N_("Chinese Simplified (_GB18030)"),      "gb18030",               LG_CHINESE },
 	{ N_("Chinese Simplified (G_B2312)"),       "GB2312",                LG_CHINESE },
 	{ N_("Chinese Simplified (GB_K)"),          "x-gbk",                 LG_CHINESE },
 	{ N_("Chinese Simplified (_HZ)"),           "HZ-GB-2312",	     LG_CHINESE },
 	{ N_("Chinese Simplified (_ISO-2022-CN)"),  "ISO-2022-CN",           LG_CHINESE },
-	{ N_("Chinese Simplified (_Windows-936)"),  "windows-936",           LG_CHINESE },
 	{ N_("Chinese Traditional (Big_5)"),        "Big5",                  LG_CHINESE },
 	{ N_("Chinese Traditional (Big5-HK_SCS)"),  "Big5-HKSCS",	     LG_CHINESE },
 	{ N_("Chinese Traditional (_EUC-TW)"),      "x-euc-tw",              LG_CHINESE },
-	{ N_("Croatian (Mac_Croatian)"),            "x-mac-croatian",        LG_CENTRAL_EUROPEAN },
 	{ N_("Cyrillic (_IBM-855)"),                "IBM855",                LG_CYRILLIC },
 	{ N_("Cyrillic (I_SO-8859-5)"),             "ISO-8859-5",	     LG_CYRILLIC },
 	{ N_("Cyrillic (IS_O-IR-111)"),             "ISO-IR-111",	     LG_CYRILLIC },
@@ -138,58 +132,54 @@ encodings[] =
 	{ N_("Cyrillic/Russian (_CP-866)"),         "IBM866",                LG_CYRILLIC },
 	{ N_("Cyrillic/Ukrainian (_KOI8-U)"),       "KOI8-U",                LG_CYRILLIC },
 	{ N_("Cyrillic/Ukrainian (Mac_Ukrainian)"), "x-mac-ukrainian",       LG_CYRILLIC },
-	{ N_("English (_US-ASCII)"),                "us-ascii",              LG_WESTERN },
-	{ N_("_Farsi (MacFarsi)"),                  "x-mac-farsi",           LG_OTHER },
-	{ N_("_Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER },
 	{ N_("Greek (_ISO-8859-7)"),                "ISO-8859-7",            LG_GREEK },
 	{ N_("Greek (_MacGreek)"),                  "x-mac-greek",           LG_GREEK },
 	{ N_("Greek (_Windows-1253)"),              "windows-1253",          LG_GREEK },
 	{ N_("Gujarati (_MacGujarati)"),            "x-mac-gujarati",        LG_INDIAN },
 	{ N_("Gurmukhi (Mac_Gurmukhi)"),            "x-mac-gurmukhi",        LG_INDIAN },
+	{ N_("Hindi (Mac_Devanagari)"),             "x-mac-devanagari",      LG_INDIAN },
 	{ N_("Hebrew (_IBM-862)"),                  "IBM862",                LG_HEBREW },
-	{ N_("Hebrew (I_SO-8859-8-E)"),             "ISO-8859-8-E",          LG_HEBREW },
 	{ N_("Hebrew (IS_O-8859-8-I)"),             "ISO-8859-8-I",          LG_HEBREW },
 	{ N_("Hebrew (_MacHebrew)"),                "x-mac-hebrew",          LG_HEBREW },
 	{ N_("Hebrew (_Windows-1255)"),             "windows-1255",          LG_HEBREW },
-	{ N_("Hindi (Mac_Devanagari)"),             "x-mac-devanagari",      LG_INDIAN },
-	{ N_("_Icelandic (MacIcelandic)"),          "x-mac-icelandic",       LG_OTHER },
+	{ N_("_Visual Hebrew (ISO-8859-8)"),        "ISO-8859-8",            LG_HEBREW },
 	{ N_("Japanese (_EUC-JP)"),                 "EUC-JP",                LG_JAPANESE },
 	{ N_("Japanese (_ISO-2022-JP)"),            "ISO-2022-JP",           LG_JAPANESE },
-	{ N_("Japanese (_Shift_JIS)"),              "Shift_JIS",             LG_JAPANESE },
+	{ N_("Japanese (_Shift-JIS)"),              "Shift_JIS",             LG_JAPANESE },
 	{ N_("Korean (_EUC-KR)"),                   "EUC-KR",                LG_KOREAN },
 	{ N_("Korean (_ISO-2022-KR)"),              "ISO-2022-KR",           LG_KOREAN },
 	{ N_("Korean (_JOHAB)"),                    "x-johab",               LG_KOREAN },
 	{ N_("Korean (_UHC)"),                      "x-windows-949",         LG_KOREAN },
-	{ N_("_Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER },
-	{ N_("_Romanian (_MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
-	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
-	{ N_("South European (_ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
-	{ N_("_Thai (TIS-620)"),                    "TIS-620",               LG_OTHER },
 	{ N_("Turkish (_IBM-857)"),                 "IBM857",                LG_TURKISH },
 	{ N_("Turkish (I_SO-8859-9)"),              "ISO-8859-9",            LG_TURKISH },
 	{ N_("Turkish (_MacTurkish)"),              "x-mac-turkish",         LG_TURKISH },
 	{ N_("Turkish (_Windows-1254)"),            "windows-1254",          LG_TURKISH },
 	{ N_("Unicode (UTF-_7)"),                   "UTF-7",                 LG_UNICODE },
 	{ N_("Unicode (UTF-_8)"),                   "UTF-8",                 LG_UNICODE },
-	{ N_("Unicode (UTF-1_6BE)"),                "UTF-16BE",              LG_UNICODE },
-	{ N_("Unicode (UTF-16_LE)"),                "UTF-16LE",              LG_UNICODE },
-	{ N_("Unicode (UTF-3_2BE)"),                "UTF-32BE",              LG_UNICODE },
-	{ N_("Unicode (UTF-32L_E)"),                "UTF-32LE",              LG_UNICODE },
-	{ N_("_User Defined"),                      "x-user-defined",        LG_OTHER },
 	{ N_("Vietnamese (_TCVN)"),                 "x-viet-tcvn5712",       LG_VIETNAMESE },
 	{ N_("Vietnamese (_VISCII)"),               "VISCII",                LG_VIETNAMESE },
 	{ N_("Vietnamese (V_PS)"),                  "x-viet-vps",            LG_VIETNAMESE },
 	{ N_("Vietnamese (_Windows-1258)"),         "windows-1258",          LG_VIETNAMESE },
-	{ N_("_Visual Hebrew (ISO-8859-8)"),        "ISO-8859-8",            LG_HEBREW },
 	{ N_("Western (_IBM-850)"),                 "IBM850",                LG_WESTERN },
 	{ N_("Western (I_SO-8859-1)"),              "ISO-8859-1",            LG_WESTERN },
 	{ N_("Western (IS_O-8859-15)"),             "ISO-8859-15",           LG_WESTERN },
 	{ N_("Western (_MacRoman)"),                "x-mac-roman",           LG_WESTERN },
 	{ N_("Western (_Windows-1252)"),            "windows-1252",          LG_WESTERN },
-	/* encodings without translatable names */
-	{ "T._61-8bit",                             "T.61-8bit",             LG_OTHER },
-	{ "x-_imap4-modified-utf7",                 "x-imap4-modified-utf7", LG_UNICODE },
-	{ "x-u-_escaped",                           "x-u-escaped",           LG_OTHER }
+	{ N_("_Armenian (ARMSCII-8)"),              "armscii-8",             LG_OTHER },
+	{ N_("_Celtic (ISO-8859-14)"),              "ISO-8859-14",           LG_OTHER },
+	{ N_("_Farsi (MacFarsi)"),                  "x-mac-farsi",           LG_OTHER },
+	{ N_("_Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER },
+	{ N_("_Icelandic (MacIcelandic)"),          "x-mac-icelandic",       LG_OTHER },
+	{ N_("_Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER },
+	{ N_("_Romanian (MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
+	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
+	{ N_("South _European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
+	{ N_("Thai (TIS-_620)"),                    "TIS-620",               LG_OTHER },
+#if MOZILLA_SNAPSHOT >= 9
+	{ N_("Thai (IS_O-8859-11)"),                "iso-8859-11",           LG_OTHER },
+	{ N_("_Thai (Windows-874)"),                "windows-874",           LG_OTHER },
+#endif	
+	{ N_("_User Defined"),                      "x-user-defined",        LG_OTHER },
 };
 static const guint n_encodings = G_N_ELEMENTS (encodings);
 
@@ -201,8 +191,7 @@ static void
 mozilla_embed_single_finalize (GObject *object);
 
 static gresult      
-impl_clear_cache (EphyEmbedSingle *shell,
-		  CacheType type);
+impl_clear_cache (EphyEmbedSingle *shell);
 static gresult          
 impl_set_offline_mode (EphyEmbedSingle *shell,
 		       gboolean offline);
@@ -253,10 +242,6 @@ static void mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 					              guint chrome_mask,
                            		              EphyEmbedSingle *shell);
 
-#ifdef GNOME_ENABLE_DEBUG
-static gresult control_encodings_list (void);
-#endif
-
 struct MozillaEmbedSinglePrivate
 {
 	char *user_prefs;
@@ -391,6 +376,13 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 
 	/* Enable Image Auto-Resizing */
 	pref->SetBoolPref ("browser.enable_automatic_image_resizing", PR_TRUE);
+
+	/* User agent */
+
+	/* FIXME We need to do this because mozilla doesnt set product
+	sub for embedding apps */
+	pref->SetCharPref ("general.useragent.vendor", "Epiphany");
+	pref->SetCharPref ("general.useragent.vendorSub", VERSION);
 }
 
 static char *
@@ -570,9 +562,6 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 
 	mozilla_register_external_protocols ();
 
-#ifdef GNOME_ENABLE_DEBUG
-	control_encodings_list ();
-#endif
 	/* FIXME alert if fails */
 }
 
@@ -649,8 +638,7 @@ mozilla_embed_single_finalize (GObject *object)
 }
 
 static gresult      
-impl_clear_cache (EphyEmbedSingle *shell,
-		  CacheType type)
+impl_clear_cache (EphyEmbedSingle *shell)
 {
 	nsresult rv;
 	
@@ -658,7 +646,7 @@ impl_clear_cache (EphyEmbedSingle *shell,
                         do_GetService (NS_CACHESERVICE_CONTRACTID, &rv);
 	if (NS_FAILED(rv)) return G_FAILED;
 
-	CacheService->EvictEntries((guint)type);
+	CacheService->EvictEntries (nsICache::STORE_ANYWHERE);
 
 	return G_OK;
 }
@@ -696,87 +684,6 @@ impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 	return G_OK;
 }
 
-#ifdef GNOME_ENABLE_DEBUG
-static gresult
-control_encodings_list (void)
-{
-	nsresult rv;
-	char *tmp;
-        PRUint32 cscount;
-        char *encoding_str, *encoding_title_str;
-	gresult ret = G_OK;
-
-        nsCOMPtr<nsIAtom> docCharsetAtom;
-        nsCOMPtr<nsICharsetConverterManager2> ccm2 =
-                do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-        if (!NS_SUCCEEDED(rv)) return G_FAILED;
-
-        nsCOMPtr <nsISupportsArray> cs_list;
-        rv = ccm2->GetDecoderList (getter_AddRefs(cs_list));
-        if (!NS_SUCCEEDED(rv)) return G_FAILED;
-
-        rv = cs_list->Count(&cscount);
-	for (PRUint32 i = 0; i < cscount; i++)
-        {
-                nsCOMPtr<nsISupports> cssupports =
-                                        (dont_AddRef)(cs_list->ElementAt(i));
-                nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
-                nsAutoString encoding_ns, encoding_title_ns;
-
-                /* encoding name */
-		rv = csatom->ToString(encoding_ns);
-                tmp = ToNewCString (encoding_ns);
-                if (tmp == NULL || strlen (tmp) == 0)
-                {
-                        continue;
-                }
-		encoding_str = g_strdup (tmp);
-		nsMemory::Free (tmp);
-		tmp = nsnull;
-
-                /* encoding readable title */
-                rv = ccm2->GetCharsetTitle2(csatom, &encoding_title_ns);
-                tmp = ToNewCString (encoding_title_ns);
-                if (tmp == NULL || 
-                    strlen (tmp) == 0)
-                {
-                        encoding_title_str = g_strdup (encoding_str);
-                }
-		else
-		{
-			encoding_title_str = g_strdup (tmp);
-		}
-
-		if (tmp) nsMemory::Free (tmp);
-		tmp = nsnull;
-
-		gboolean found = FALSE;
-		for (PRUint32 j = 0; j < n_encodings; j++)
-                {
-                        if (g_ascii_strcasecmp (
-                                encoding_str, 
-                                encodings[j].name) == 0)
-                        {
-				LOG ("Mozilla reported encoding %s with title %s found in our list.",
-				     encoding_str, encoding_title_str)
-
-				found = TRUE;
-                                break;
-                        }
-                }
-
-		if (found == FALSE)
-		{
-			g_warning ("Mozilla reported encoding %s with title %s NOT found in our list!",
-				   encoding_str, encoding_title_str);
-			ret = G_FAILED;
-		}
-        }
-
-	return ret;
-}
-#endif
-
 static gint
 encoding_info_cmp (const EncodingInfo *i1, const EncodingInfo *i2)
 {
@@ -803,7 +710,7 @@ impl_get_encodings (EphyEmbedSingle *shell,
 
 			info->encoding = g_strdup (encodings[i].name);
 			
-			elided = ephy_str_elide_underscores (_(encodings[i].title));
+			elided = ephy_string_elide_underscores (_(encodings[i].title));
 
 			if (elide_underscores)
 			{
@@ -853,7 +760,7 @@ impl_get_language_groups (EphyEmbedSingle *shell,
 		info->group = lang_groups[i].group;
 		
 		/* collate without underscores */
-		elided = ephy_str_elide_underscores (info->title);
+		elided = ephy_string_elide_underscores (info->title);
 		info->key = g_utf8_collate_key (elided, -1);
 		g_free (elided);
 
@@ -999,9 +906,9 @@ impl_remove_cookies (EphyEmbedSingle *shell,
         {
                 CookieInfo *c = (CookieInfo *)cl->data;
 
-                result = cookieManager->Remove (NS_LITERAL_CSTRING(c->domain),
-                                                NS_LITERAL_CSTRING(c->name),
-                                                NS_LITERAL_CSTRING(c->path),
+                result = cookieManager->Remove (nsDependentCString(c->domain),
+                                                nsDependentCString(c->name),
+                                                nsDependentCString(c->path),
                                                 PR_FALSE);
                 if (NS_FAILED(result)) return G_FAILED;
         };
@@ -1122,7 +1029,10 @@ impl_show_file_picker (EphyEmbedSingle *shell,
         g_free (expanded_directory);
 
         filePicker->InitWithGtkWidget (parentWidget, title, mode);
-        filePicker->SetDefaultString (NS_ConvertUTF8toUCS2(file).get());
+	if (file)
+	{
+	        filePicker->SetDefaultString (NS_ConvertUTF8toUCS2(file).get());
+	}
         filePicker->SetDisplayDirectory (dir);
         
         PRInt16 retval;
