@@ -206,6 +206,9 @@ mozilla_embed_security_change_cb (GtkMozEmbed *embed,
                                   guint state, MozillaEmbed *membed);
 static EmbedSecurityLevel
 mozilla_embed_security_level (MozillaEmbed *membed);
+static gint
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed);
 
 /* signals to connect on each embed widget */
 static const struct
@@ -228,6 +231,7 @@ signal_connections[] =
         { "size_to",         (void *) mozilla_embed_size_to_cb           },
         { "new_window",      (void *) mozilla_embed_new_window_cb        },
         { "security_change", (void *) mozilla_embed_security_change_cb   },
+	{ "dom_key_down",    (void *) mozilla_embed_dom_key_down_cb      },
 
         /* terminator -- must be last in the list! */
         { NULL, NULL } 
@@ -349,7 +353,7 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 static void
 mozilla_embed_init (MozillaEmbed *embed)
 {
-        embed->priv = new MozillaEmbedPrivate;
+        embed->priv = g_new0 (MozillaEmbedPrivate, 1);
 
 	mozilla_embed_connect_signals (embed);
 }
@@ -502,7 +506,7 @@ impl_can_go_up (EphyEmbed *embed)
 	{
 		g_free (s);
 		result = G_OK;
-	}				
+	}
 	else
 	{
 		result = G_FAILED;
@@ -562,12 +566,13 @@ impl_go_up (EphyEmbed *embed)
 	g_return_val_if_fail (uri != NULL, G_FAILED);
 	
 	parent_uri = mozilla_embed_get_uri_parent (uri);
+	g_free (uri);
 	g_return_val_if_fail (parent_uri != NULL, G_FAILED);
 	
 	ephy_embed_load_url (embed, parent_uri);
-	
+
 	g_free (parent_uri);
-	
+
 	return G_OK;
 }
 
@@ -699,8 +704,9 @@ impl_get_location (EphyEmbed *embed,
 	
 	if (toplevel)
 	{
-		l = gtk_moz_embed_get_location 
-			(GTK_MOZ_EMBED(embed));
+		rv = wrapper->GetMainDocumentUrl (url);
+		l = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ?
+		     g_strdup (url.get()) : NULL;	   	
 	}
 	else
 	{
@@ -1079,7 +1085,7 @@ impl_print (EphyEmbed *embed,
 
         nsCOMPtr<nsIPrintSettings> options;
         result = wrapper->GetPrintSettings(getter_AddRefs(options));
-        g_assert (NS_SUCCEEDED (result));
+        if (!NS_SUCCEEDED (result)) return G_FAILED;
 
 	MozillaCollatePrintSettings(info, options);
 
@@ -1182,7 +1188,11 @@ mozilla_embed_location_changed_cb (GtkMozEmbed *embed,
 	 * to know about it. */
 	if (membed->priv->no_page != 0)
 	{
-		g_signal_emit_by_name (membed, "ge_location");
+		char *location;
+
+		location = gtk_moz_embed_get_location (embed);
+		g_signal_emit_by_name (membed, "ge_location", location);
+		g_free (location);
 	}
 
 	membed->priv->no_page = -1;
@@ -1311,6 +1321,63 @@ mozilla_embed_visibility_cb (GtkMozEmbed *embed, gboolean visibility,
 	}
 }
 
+static gint
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed)
+{
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_key_down_cb: domevent NULL");
+		return FALSE;
+	}
+
+	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
+	if (!ev)
+	{
+		return FALSE;
+	}
+
+	EphyWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+
+	EphyEmbedEvent *info;
+	info = ephy_embed_event_new ();
+
+	gboolean ret = FALSE;
+
+	nsresult rv;
+	EventContext ctx;
+	ctx.Init (wrapper);
+	rv = ctx.GetKeyEventInfo (ev, info);
+	if (NS_SUCCEEDED(rv) &&
+	    (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	     info->modifier == GDK_SHIFT_MASK))
+	{
+		/* Translate relative coordinates to absolute values, and try
+		   to avoid covering links by adding a little offset. */
+
+		int x, y;
+		gdk_window_get_origin (GTK_WIDGET(membed)->window, &x, &y);
+		info->x += x + 6;	
+		info->y += y + 6;
+
+		nsCOMPtr<nsIDOMDocument> doc;
+		rv = ctx.GetTargetDocument (getter_AddRefs(doc));
+		if (NS_SUCCEEDED(rv))
+		{
+			rv = wrapper->PushTargetDocument (doc);
+			if (NS_SUCCEEDED(rv))
+			{
+				g_signal_emit_by_name (membed, "ge_context_menu", info, &ret);
+				wrapper->PopTargetDocument ();
+			}
+		}
+	}
+
+	g_object_unref (info);
+	return ret;
+}
+
 static void
 mozilla_embed_destroy_brsr_cb (GtkMozEmbed *embed, 
 			       MozillaEmbed *membed)
@@ -1319,9 +1386,8 @@ mozilla_embed_destroy_brsr_cb (GtkMozEmbed *embed,
 }
 
 static gint
-mozilla_embed_emit_mouse_signal (MozillaEmbed *membed,
-				 gpointer dom_event,
-				 const char *signal_name)
+mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, 
+				  MozillaEmbed *membed)
 {
 	EphyEmbedEvent *info;
 	EventContext event_context;
@@ -1329,13 +1395,20 @@ mozilla_embed_emit_mouse_signal (MozillaEmbed *membed,
 	EphyWrapper *wrapper;
 	nsresult result;
 
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_mouse_click_cb: domevent NULL");
+		return FALSE;
+	}
+
 	info = ephy_embed_event_new ();
 	
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
+	event_context.Init (wrapper);
+        result = event_context.GetMouseEventInfo (static_cast<nsIDOMMouseEvent*>(dom_event), info);
+
 	if (NS_SUCCEEDED(result))
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
@@ -1345,7 +1418,7 @@ mozilla_embed_emit_mouse_signal (MozillaEmbed *membed,
 			result = wrapper->PushTargetDocument (domDoc);
 			if (NS_SUCCEEDED(result))
 			{
-				g_signal_emit_by_name (membed, signal_name, 
+				g_signal_emit_by_name (membed, "ge_dom_mouse_click", 
 						       info, &return_value); 
 				wrapper->PopTargetDocument ();
 			}
@@ -1354,24 +1427,57 @@ mozilla_embed_emit_mouse_signal (MozillaEmbed *membed,
 	}
 
 	g_object_unref (info);
-	
-	return return_value;
-}
 
-static gint
-mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, 
-				  MozillaEmbed *membed)
-{
-	return mozilla_embed_emit_mouse_signal
-		(membed, dom_event, "ge_dom_mouse_click");
+	return return_value;
 }
 
 static gint
 mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
 				 MozillaEmbed *membed)
 {
-	return mozilla_embed_emit_mouse_signal
-		(membed, dom_event, "ge_dom_mouse_down");
+	EphyEmbedEvent *info;
+	EventContext event_context;
+	gint return_value = 0;
+	EphyWrapper *wrapper;
+	nsresult result;
+	EphyEmbedEventType type;
+
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_mouse_down_cb: domevent NULL");
+		return FALSE;
+	}
+
+	info = ephy_embed_event_new ();
+	
+	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+
+	event_context.Init (wrapper);
+        result = event_context.GetMouseEventInfo (static_cast<nsIDOMMouseEvent*>(dom_event), info);
+
+	ephy_embed_event_get_event_type (info, &type);
+
+	if (NS_SUCCEEDED(result) && (type == EPHY_EMBED_EVENT_MOUSE_BUTTON3))
+	{
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+		if (NS_SUCCEEDED(result))
+		{
+			result = wrapper->PushTargetDocument (domDoc);
+			if (NS_SUCCEEDED(result))
+			{
+				g_signal_emit_by_name (membed, "ge_context_menu", 
+						       info, &return_value); 
+				wrapper->PopTargetDocument ();
+			}
+		}
+
+	}
+
+	g_object_unref (info);
+
+	return return_value;
 }
 
 static void
@@ -1401,7 +1507,7 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 		{ GTK_MOZ_EMBED_FLAG_DEFAULTCHROME, EMBED_CHROME_DEFAULT },
 		{ GTK_MOZ_EMBED_FLAG_MENUBARON, EMBED_CHROME_MENUBARON },
 		{ GTK_MOZ_EMBED_FLAG_TOOLBARON, EMBED_CHROME_TOOLBARON },
-		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_BOOKMARKSBARON },
+		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_BOOKMARKSBAR_DEFAULT },
 		{ GTK_MOZ_EMBED_FLAG_STATUSBARON, EMBED_CHROME_STATUSBARON },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWRAISED, EMBED_CHROME_WINDOWRAISED },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWLOWERED, EMBED_CHROME_WINDOWLOWERED },
