@@ -37,7 +37,6 @@
 #include "ephy-history-window.h"
 #include "ephy-shell.h"
 #include "ephy-dnd.h"
-#include "ephy-prefs.h"
 #include "egg-action-group.h"
 #include "egg-toggle-action.h"
 #include "egg-menu-merge.h"
@@ -50,6 +49,7 @@
 #include "ephy-gui.h"
 #include "toolbar.h"
 #include "ephy-stock-icons.h"
+#include "ephy-search-entry.h"
 
 static GtkTargetEntry page_drag_types [] =
 {
@@ -73,8 +73,6 @@ static void ephy_history_window_get_property (GObject *object,
 					      GParamSpec *pspec);
 static void ephy_history_window_dispose      (GObject *object);
 
-static void search_entry_changed_cb       (GtkWidget *entry,
-					   EphyHistoryWindow *editor);
 static void cmd_open_bookmarks_in_tabs    (EggAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_open_bookmarks_in_browser (EggAction *action,
@@ -217,21 +215,46 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *image;
+	GtkWidget *button;
+	GtkWidget *align;
 	char *str;
 
 	dialog = gtk_dialog_new_with_buttons (_("Clear history"),
 					     GTK_WINDOW (editor),
 					     GTK_DIALOG_DESTROY_WITH_PARENT |
 					     GTK_DIALOG_NO_SEPARATOR,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_CLEAR,
-					     GTK_RESPONSE_OK,
 					     NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
 
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CANCEL);
+
+	button = gtk_button_new ();
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_widget_show (align);
+	gtk_container_add (GTK_CONTAINER (button), align);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (align), hbox);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+	label = gtk_label_new_with_mnemonic (_("C_lear"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
@@ -781,20 +804,6 @@ key_pressed_cb (EphyNodeView *view,
 	return FALSE;
 }
 
-static void
-reset_search_entry (EphyHistoryWindow *editor)
-{
-	g_signal_handlers_block_by_func
-		(G_OBJECT (editor->priv->search_entry),
-		 G_CALLBACK (search_entry_changed_cb),
-		 editor);
-	gtk_entry_set_text (GTK_ENTRY (editor->priv->search_entry), "");
-	g_signal_handlers_unblock_by_func
-		(G_OBJECT (editor->priv->search_entry),
-		 G_CALLBACK (search_entry_changed_cb),
-		 editor);
-}
-
 static void
 site_node_selected_cb (EphyNodeView *view,
 		       EphyNode *node,
@@ -809,16 +818,15 @@ site_node_selected_cb (EphyNodeView *view,
 	}
 	else
 	{
-		reset_search_entry (editor);
+		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
 		pages_filter (editor, node);
 	}
 }
 
 static void
-search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
+search_entry_search_cb (GtkWidget *entry, char *search_text, EphyHistoryWindow *editor)
 {
 	EphyNode *all;
-	char *search_text;
 
 	g_signal_handlers_block_by_func
 		(G_OBJECT (editor->priv->sites_view),
@@ -832,8 +840,6 @@ search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
 		 G_CALLBACK (site_node_selected_cb),
 		 editor);
 
-	search_text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-
 	GDK_THREADS_ENTER ();
 
 	ephy_node_filter_empty (editor->priv->pages_filter);
@@ -850,8 +856,6 @@ search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
 	ephy_node_filter_done_changing (editor->priv->pages_filter);
 
 	GDK_THREADS_LEAVE ();
-
-	g_free (search_text);
 }
 
 static GtkWidget *
@@ -866,13 +870,13 @@ build_search_box (EphyHistoryWindow *editor)
 	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 	gtk_widget_show (box);
 
-	entry = gtk_entry_new ();
+	entry = ephy_search_entry_new ();
 	add_focus_monitor (editor, entry);
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
 	gtk_widget_show (entry);
-	g_signal_connect (G_OBJECT (entry), "changed",
-			  G_CALLBACK (search_entry_changed_cb),
+	g_signal_connect (G_OBJECT (entry), "search",
+			  G_CALLBACK (search_entry_search_cb),
 			  editor);
 
 	label = gtk_label_new (NULL);
