@@ -31,12 +31,9 @@ static EphyEmbedEvent *
 get_event_info (EphyWindow *window)
 {
 	EphyEmbedEvent *info;
-	EphyTab *tab;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_val_if_fail (tab != NULL, NULL);
-
-	info = ephy_tab_get_event (tab);
+	info = EPHY_EMBED_EVENT (g_object_get_data
+		(G_OBJECT (window), "context_event"));
 	g_return_val_if_fail (info != NULL, NULL);
 
 	return info;
@@ -164,11 +161,10 @@ popup_cmd_bookmark_link (EggAction *action,
 					 location))
 	{
 		new_bookmark = ephy_new_bookmark_new
-			(bookmarks, GTK_WINDOW (window), location);
+			(bookmarks, GTK_WINDOW (window),
+			 is_smart ? rel : location);
 		ephy_new_bookmark_set_title
 			(EPHY_NEW_BOOKMARK (new_bookmark), title);
-		ephy_new_bookmark_set_smarturl
-			(EPHY_NEW_BOOKMARK (new_bookmark), rel);
 		gtk_widget_show (new_bookmark);
 	}
 }
@@ -227,39 +223,28 @@ popup_cmd_copy_to_clipboard (EphyWindow *window, const char *text)
 }
 
 void
-popup_cmd_copy_email (EggAction *action,
-                      EphyWindow *window)
+popup_cmd_copy_link_address (EggAction *action,
+			     EphyWindow *window)
 {
-	EphyEmbedEvent *info;
-	const char *location;
+	EphyEmbedEvent *event;
+	const char *address;
 	const GValue *value;
-	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
+	event = get_event_info (window);
+	g_return_if_fail (IS_EPHY_EMBED_EVENT (event));
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "email", &value);
-	location = g_value_get_string (value);
-	popup_cmd_copy_to_clipboard (window, location);
-}
-
-void
-popup_cmd_copy_link_location (EggAction *action,
-			      EphyWindow *window)
-{
-	EphyEmbedEvent *info;
-	const char *location;
-	const GValue *value;
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "link", &value);
-	location = g_value_get_string (value);
-	popup_cmd_copy_to_clipboard (window, location);
+	if (event->context & EMBED_CONTEXT_EMAIL_LINK)
+	{
+		ephy_embed_event_get_property (event, "email", &value);
+		address = g_value_get_string (value);
+		popup_cmd_copy_to_clipboard (window, address);
+	}
+	else if (event->context & EMBED_CONTEXT_LINK)
+	{
+		ephy_embed_event_get_property (event, "link", &value);
+		address = g_value_get_string (value);
+		popup_cmd_copy_to_clipboard (window, address);
+	}
 }
 
 static void
@@ -440,6 +425,8 @@ popup_cmd_open_frame (EggAction *action,
 	ephy_embed_get_location (embed, FALSE, &location);
 
 	ephy_embed_load_url (embed, location);
+
+	g_free (location);
 }
 
 void
@@ -460,4 +447,3 @@ popup_cmd_open_image (EggAction *action,
 
 	ephy_embed_load_url (embed, location);
 }
-
