@@ -54,11 +54,8 @@ void
 prefs_proxy_auto_url_reload_cb (GtkWidget *button,
 			        EphyDialog *dialog);
 void
-prefs_clear_memory_cache_button_clicked_cb (GtkWidget *button,
-					    gpointer data);
-void
-prefs_clear_disk_cache_button_clicked_cb (GtkWidget *button,
-					  gpointer data);
+prefs_clear_cache_button_clicked_cb (GtkWidget *button,
+				     gpointer data);
 void
 prefs_dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer data);
 void
@@ -91,6 +88,8 @@ languages [] =
 	{ N_("Bulgarian"), "bg" },
 	{ N_("Byelorussian"), "be" },
 	{ N_("Catalan"), "ca" },
+	{ N_("Simplified Chinese"), "zh-cn" },
+	{ N_("Traditional Chinese"), "zh-tw" },
 	{ N_("Chinese"), "zh" },
 	{ N_("Croatian"), "hr" },
 	{ N_("Czech"), "cs" },
@@ -118,11 +117,11 @@ languages [] =
 	{ N_("Macedonian"), "mk" },
 	{ N_("Malay"), "ms" },
 	{ N_("Norwegian/Nynorsk"), "nn" },
-	{ N_("Norwegian/Bokmaal"), "nb" },
+	{ N_("Norwegian/Bokmal"), "nb" },
 	{ N_("Norwegian"), "no" },
 	{ N_("Polish"), "pl" },
 	{ N_("Portuguese"), "pt" },
-	{ N_("Portuguese of Brazil"), "pt-BR" },
+	{ N_("Portuguese of Brazil"), "pt-br" },
 	{ N_("Romanian"), "ro" },
 	{ N_("Russian"), "ru" },
 	{ N_("Scottish"), "gd" },
@@ -134,7 +133,7 @@ languages [] =
 	{ N_("Tamil"), "ta" },
 	{ N_("Turkish"), "tr" },
 	{ N_("Ukrainian"), "uk" },
-	{ N_("Vietnamian"), "vi" },
+	{ N_("Vietnamese"), "vi" },
 	{ N_("Walloon"), "wa" }
 };
 static guint n_languages = G_N_ELEMENTS (languages);
@@ -149,16 +148,38 @@ static EncodingAutodetectorInfo encoding_autodetector[] =
 {
 	{ N_("Off"),			"" },
 	{ N_("Chinese"),		"zh_parallel_state_machine" },
-	{ N_("East asian"),		"cjk_parallel_state_machine" },
+	{ N_("East Asian"),		"cjk_parallel_state_machine" },
 	{ N_("Japanese"),		"ja_parallel_state_machine" },
 	{ N_("Korean"),			"ko_parallel_state_machine" },
 	{ N_("Russian"),		"ruprob" },
 	{ N_("Simplified Chinese"),	"zhcn_parallel_state_machine" },
 	{ N_("Traditional Chinese"),	"zhtw_parallel_state_machine" },
+	{ N_("Universal"),		"universal_charset_detector" },
 	{ N_("Ukrainian"),		"ukprob" }
 };
 static guint n_encoding_autodetectors = G_N_ELEMENTS (encoding_autodetector);
 
+static const
+char *cookies_accept_enum [] =
+{
+	"anywhere", "current site", "nowhere"
+};
+static guint n_cookies_accept_enum = G_N_ELEMENTS (cookies_accept_enum);
+
+static const
+char *cache_compare_enum [] =
+{
+	"once per session", "every time", "never", "automatic"
+};
+static guint n_cache_compare_enum = G_N_ELEMENTS (cache_compare_enum);
+
+static const
+char *proportional_enum [] =
+{
+	"serif", "sans-serif"
+};
+static guint n_proportional_enum = G_N_ELEMENTS (proportional_enum);
+
 enum
 {
 	FONT_TYPE_SERIF,
@@ -197,11 +218,8 @@ enum
 	/* General */
 	OPEN_IN_TABS_PROP,
 	HOMEPAGE_ENTRY_PROP,
-	AUTO_ENCODING_PROP,
-	DEFAULT_ENCODING_PROP,
-	LANGUAGE_PROP,
 
-	/* Appeareance */
+	/* Fonts and Colors */
 	FONTS_LANGUAGE_PROP,
 	SERIF_PROP,
 	SANSSERIF_PROP,
@@ -222,9 +240,13 @@ enum
 	/* Advanced */
 	CACHE_COMPARE_PROP,
 	DISK_CACHE_PROP,
-	MEMORY_CACHE_PROP
+	AUTO_ENCODING_PROP,
+	DEFAULT_ENCODING_PROP,
+	LANGUAGE_PROP,
 };
 
+#define CONF_FONTS_FOR_LANGUAGE	"/apps/epiphany/dialogs/preferences_font_language"
+
 static const
 EphyDialogProperty properties [] =
 {
@@ -234,12 +256,9 @@ EphyDialogProperty properties [] =
 	/* General */
 	{ OPEN_IN_TABS_PROP, "open_in_tabs_checkbutton", CONF_TABS_TABBED, PT_AUTOAPPLY, NULL },
 	{ HOMEPAGE_ENTRY_PROP, "homepage_entry", CONF_GENERAL_HOMEPAGE, PT_AUTOAPPLY, NULL },
-	{ AUTO_ENCODING_PROP, "auto_encoding_optionmenu", NULL, PT_NORMAL, NULL },
-	{ DEFAULT_ENCODING_PROP, "default_encoding_optionmenu", NULL, PT_NORMAL, NULL },
-	{ LANGUAGE_PROP, "language_optionmenu", NULL, PT_NORMAL, NULL },
 
-	/* Appeareance */
-	{ FONTS_LANGUAGE_PROP, "fonts_language_optionmenu", NULL, PT_NORMAL, NULL },
+	/* Fonts and Colors */
+	{ FONTS_LANGUAGE_PROP, "fonts_language_optionmenu", CONF_FONTS_FOR_LANGUAGE, PT_AUTOAPPLY, NULL },
 	{ SERIF_PROP, "serif_combo", NULL, PT_NORMAL, NULL },
 	{ SANSSERIF_PROP, "sansserif_combo", NULL, PT_NORMAL, NULL },
 	{ MONOSPACE_PROP, "monospace_combo", NULL, PT_NORMAL, NULL },
@@ -258,8 +277,10 @@ EphyDialogProperty properties [] =
 
 	/* Advanced */
 	{ CACHE_COMPARE_PROP, "cache_compare_radiobutton", CONF_NETWORK_CACHE_COMPARE, PT_AUTOAPPLY, NULL },
-	{ DISK_CACHE_PROP, "disk_cache_spin", CONF_NETWORK_DISK_CACHE, PT_AUTOAPPLY, NULL },
-	{ MEMORY_CACHE_PROP, "memory_cache_spin", CONF_NETWORK_MEMORY_CACHE, PT_AUTOAPPLY, NULL },
+	{ DISK_CACHE_PROP, "disk_cache_spin", CONF_NETWORK_CACHE_SIZE, PT_AUTOAPPLY, NULL },
+	{ AUTO_ENCODING_PROP, "auto_encoding_optionmenu", NULL, PT_NORMAL, NULL },
+	{ DEFAULT_ENCODING_PROP, "default_encoding_optionmenu", NULL, PT_NORMAL, NULL },
+	{ LANGUAGE_PROP, "language_optionmenu", NULL, PT_NORMAL, NULL },
 
 	{ -1, NULL, NULL }
 };
@@ -395,7 +416,7 @@ prefs_dialog_show_help (PrefsDialog *pd)
 
 	id = gtk_notebook_get_current_page (GTK_NOTEBOOK (pd->priv->notebook));
 
-	ephy_gui_help (GTK_WINDOW (pd), "epiphany", help_preferences[id]);
+	ephy_gui_help (GTK_WINDOW (pd->priv->window), "epiphany", help_preferences[id]);
 }
 
 static const gchar *
@@ -432,8 +453,8 @@ setup_font_menu (PrefsDialog *dialog,
 					 type, &fonts, &default_font);
 
 	/* Get the default font */
-	snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, type,
-		  get_current_language_code (dialog));
+	g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, type,
+		    get_current_language_code (dialog));
 	name = eel_gconf_get_string (key);
 	if (name == NULL)
 	{
@@ -481,9 +502,9 @@ save_font_menu (PrefsDialog *dialog,
 		return;
 	}
 
-	snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT,
-		  fonts_types[type],
-		  get_current_language_code (dialog));
+	g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT,
+		    fonts_types[type],
+		    get_current_language_code (dialog));
 	eel_gconf_set_string (key, name);
 	g_free (name);
 }
@@ -537,9 +558,9 @@ size_spinbutton_changed_cb (GtkWidget *spin, PrefsDialog *dialog)
 
 	type = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(spin), "type"));
 
-	snprintf (key, 255, "%s_%s",
-		  size_prefs[type],
-		  get_current_language_code (dialog));
+	g_snprintf (key, 255, "%s_%s",
+		    size_prefs[type],
+		    get_current_language_code (dialog));
 	eel_gconf_set_integer (key, gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin)));
 }
 
@@ -582,8 +603,8 @@ setup_size_control (PrefsDialog *dialog,
 	char key[255];
 	int size;
 
-	snprintf (key, 255, "%s_%s", pref,
-		  get_current_language_code (dialog));
+	g_snprintf (key, 255, "%s_%s", pref,
+		    get_current_language_code (dialog));
 	size = eel_gconf_get_integer (key);
 
 	if (size == 0) size = default_size;
@@ -643,6 +664,12 @@ create_fonts_language_menu (PrefsDialog *dialog)
 	GtkWidget *optionmenu, *menu;
 	GList *l = NULL;
 	guint i;
+	guint n_fonts_languages;
+	const FontsLanguageInfo *fonts_language;
+	char **lang_codes;
+
+	n_fonts_languages = ephy_langs_get_n_font_languages ();
+	fonts_language = ephy_langs_get_font_languages ();
 
 	for (i = 0; i < n_fonts_languages; i++)
 	{
@@ -675,9 +702,23 @@ create_fonts_language_menu (PrefsDialog *dialog)
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(optionmenu), menu);
 
-	/* FIXME: find a way to set this to the user's current locale's lang */
-	gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu),
-				     dialog->priv->language);
+	lang_codes = g_new0 (char *, n_fonts_languages);
+	i = 0;
+	for (l = dialog->priv->fonts_languages; l != NULL; l = l->next)
+	{
+		FontsLanguageInfo *info = (FontsLanguageInfo *) l->data;
+
+		lang_codes[i] = info->code;
+		
+		i++;
+	}
+	ephy_dialog_add_enum (EPHY_DIALOG (dialog), FONTS_LANGUAGE_PROP,
+			      n_fonts_languages, (const char **) lang_codes);
+	g_free (lang_codes);
+
+	dialog->priv->language =
+		gtk_option_menu_get_history (GTK_OPTION_MENU (optionmenu));
+
 	g_signal_connect (optionmenu, "changed",
 			  G_CALLBACK (fonts_language_optionmenu_changed_cb),
 			  dialog);
@@ -983,22 +1024,23 @@ create_language_menu (PrefsDialog *dialog)
 
 	/* init value from first element of the list */
 	list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
-	g_return_if_fail (list != NULL); /* FIXME: doesn't connect the handler! */
-
-	code = (const gchar *) list->data;
-	lang = g_list_find_custom (dialog->priv->langs, code,
-				   (GCompareFunc)find_lang_code);
-
-	if (lang)
+	if (list)
 	{
-		i = g_list_position (dialog->priv->langs, lang);
+		code = (const gchar *) list->data;
+		lang = g_list_find_custom (dialog->priv->langs, code,
+					   (GCompareFunc)find_lang_code);
+
+		if (lang)
+		{
+			i = g_list_position (dialog->priv->langs, lang);
+		}
+
+		g_slist_foreach (list, (GFunc) g_free, NULL);
+		g_slist_free (list);
 	}
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU(optionmenu), i);
 
-	g_slist_foreach (list, (GFunc) g_free, NULL);
-	g_slist_free (list);
-
 	g_signal_connect (optionmenu, "changed",
 			  G_CALLBACK (language_menu_changed_cb),
 			  dialog);
@@ -1033,6 +1075,13 @@ prefs_dialog_init (PrefsDialog *pd)
 			       "prefs-dialog.glade",
 			       "prefs_dialog");
 
+	ephy_dialog_add_enum (EPHY_DIALOG (pd), CACHE_COMPARE_PROP,
+			      n_cache_compare_enum, cache_compare_enum);
+	ephy_dialog_add_enum (EPHY_DIALOG (pd), ACCEPT_COOKIES_PROP,
+			      n_cookies_accept_enum, cookies_accept_enum);
+	ephy_dialog_add_enum (EPHY_DIALOG (pd), PROPORTIONAL_PROP,
+			      n_proportional_enum, proportional_enum);
+
 	pd->priv->window = ephy_dialog_get_control (dialog, WINDOW_PROP);
 	pd->priv->notebook = ephy_dialog_get_control (dialog, NOTEBOOK_PROP);
 	pd->priv->langs = NULL;
@@ -1076,27 +1125,15 @@ prefs_dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer data)
 }
 
 void
-prefs_clear_memory_cache_button_clicked_cb (GtkWidget *button,
-					    gpointer data)
+prefs_clear_cache_button_clicked_cb (GtkWidget *button,
+				     gpointer data)
 {
 	EphyEmbedSingle *single;
 
 	single = ephy_embed_shell_get_embed_single
 		(EPHY_EMBED_SHELL (ephy_shell));
 
-	ephy_embed_single_clear_cache (single, MEMORY_CACHE);
-}
-
-void
-prefs_clear_disk_cache_button_clicked_cb (GtkWidget *button,
-					  gpointer data)
-{
-	EphyEmbedSingle *single;
-
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
-	ephy_embed_single_clear_cache (single, DISK_CACHE);
+	ephy_embed_single_clear_cache (single);
 }
 
 void
@@ -1120,15 +1157,18 @@ prefs_homepage_current_button_clicked_cb (GtkWidget *button,
 					  EphyDialog *dialog)
 {
 	EphyWindow *window;
-	EphyTab *tab;
+	EphyEmbed *embed;
+	char *location;
 
 	window = ephy_shell_get_active_window (ephy_shell);
 	g_return_if_fail (window != NULL);
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	set_homepage_entry (dialog, ephy_tab_get_location (tab));
+	ephy_embed_get_location (embed, TRUE, &location);
+	set_homepage_entry (dialog, location);
+	g_free (location);
 }
 
 void
