@@ -173,10 +173,11 @@ get_session_filename (const char *filename)
 static void
 do_session_resume (Session *session)
 {
-	const char *crashed_session;
+	char *crashed_session;
 
 	crashed_session = get_session_filename (SESSION_CRASHED);
 	session_load (session, crashed_session);
+	g_free (crashed_session);
 }
 
 static void
@@ -444,9 +445,10 @@ save_tab (EphyWindow *window,
 	  xmlNodePtr window_node)
 {
 	EmbedChromeMask chrome;
-	const char *location;
+	char *location;
 	const char *title;
         xmlNodePtr embed_node;
+	EphyEmbed *embed;
 
 	chrome = ephy_window_get_chrome (window);
 
@@ -462,8 +464,10 @@ save_tab (EphyWindow *window,
 	xmlSetProp (embed_node, "title", title);
 
         /* otherwise, use the actual location. */
-	location = ephy_tab_get_location (tab);
+	embed = ephy_tab_get_embed (tab);
+	ephy_embed_get_location (embed, TRUE, &location);
         xmlSetProp (embed_node, "url", location);
+	g_free (location);
 
 	/* insert node into the tree */
 	xmlAddChild (window_node, embed_node);
@@ -518,14 +522,14 @@ session_save (Session *session,
                 gtk_window_get_position (GTK_WINDOW(wmain), &x, &y);
 
                 /* set window properties */
-                snprintf(buffer, 32, "%d", x);
+                g_snprintf(buffer, 32, "%d", x);
                 xmlSetProp (window_node, "x", buffer);
-                snprintf(buffer, 32, "%d", y);
+                g_snprintf(buffer, 32, "%d", y);
 
 		xmlSetProp (window_node, "y", buffer);
-                snprintf(buffer, 32, "%d", width);
+                g_snprintf(buffer, 32, "%d", width);
                 xmlSetProp (window_node, "width", buffer);
-                snprintf(buffer, 32, "%d", height);
+                g_snprintf(buffer, 32, "%d", height);
                 xmlSetProp (window_node, "height", buffer);
 
 		for (l = tabs; l != NULL; l = l->next)
@@ -607,16 +611,16 @@ session_load (Session *session,
 			xmlChar *tmp;
 
 			tmp = xmlGetProp (child, "x");
-			ephy_str_to_int (tmp, &x);
+			ephy_string_to_int (tmp, &x);
 			xmlFree (tmp);
 			tmp = xmlGetProp (child, "y");
-			ephy_str_to_int (tmp, &y);
+			ephy_string_to_int (tmp, &y);
 			xmlFree (tmp);
 			tmp = xmlGetProp (child, "width");
-			ephy_str_to_int (tmp, &width);
+			ephy_string_to_int (tmp, &width);
 			xmlFree (tmp);
 			tmp = xmlGetProp (child, "height");
-			ephy_str_to_int (tmp, &height);
+			ephy_string_to_int (tmp, &height);
 			xmlFree (tmp);
 
 			window = ephy_window_new ();
