@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.66 2003/06/23 09:57:21 mpeseng Exp $
+ *  $Id: window-commands.c,v 1.69 2003/06/27 19:09:51 mpeseng Exp $
  */
 
 #include <config.h>
@@ -156,16 +156,11 @@ void
 window_cmd_go_home (EggAction *action,
 		    EphyWindow *window)
 {
-	EphyEmbed *embed;
 	char *location;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
 	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
-	g_return_if_fail (location != NULL);
 
-	ephy_embed_load_url (embed, location);
+	ephy_window_load_url (window, location ? location : "about:blank");
 
 	g_free (location);
 }
